/*
 * Project: MoleCuilder
 * Description: creates and alters molecular systems
 * Copyright (C)  2010 University of Bonn. All rights reserved.
 * Please see the LICENSE file or "Copyright notice" in builder.cpp for details.
 */

/*
 * ElementDbAction.cpp
 *
 *  Created on: May 9, 2010
 *      Author: heber
 */

// include config.h
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "Helpers/MemDebug.hpp"

#include "Actions/CommandAction/ElementDbAction.hpp"
#include "Actions/ActionRegistry.hpp"
#include "config.hpp"
#include "Helpers/Log.hpp"
#include "periodentafel.hpp"
#include "Helpers/Verbose.hpp"
#include "World.hpp"

#include <iostream>
#include <string>

using namespace std;

#include "UIElements/UIFactory.hpp"
#include "UIElements/Dialog.hpp"
#include "Actions/ValueStorage.hpp"

const char CommandElementDbAction::NAME[] = "element-db";

CommandElementDbAction::CommandElementDbAction() :
  Action(NAME)
{}

CommandElementDbAction::~CommandElementDbAction()
{}

void CommandElementDb(std::string &databasepath) {
  ValueStorage::getInstance().setCurrentValue(CommandElementDbAction::NAME, databasepath);
  ActionRegistry::getInstance().getActionByName(CommandElementDbAction::NAME)->call(Action::NonInteractive);
};

void CommandElementDbAction::getParametersfromValueStorage()
{};

Dialog* CommandElementDbAction::fillDialog(Dialog *dialog) {
  ASSERT(dialog,"No Dialog given when filling action dialog");

  dialog->queryString(NAME, ValueStorage::getInstance().getDescription(NAME));

  return dialog;
}

Action::state_ptr CommandElementDbAction::performCall() {
  ostringstream usage;
  string databasepath;

  // get the path
  // TODO: Make databasepath a std::string
  config *configuration = World::getInstance().getConfig();
  ValueStorage::getInstance().queryCurrentValue(NAME, databasepath);
  strcpy(configuration->databasepath, databasepath.c_str());

  // load table
  periodentafel *periode = World::getInstance().getPeriode();
  if (periode->LoadPeriodentafel(configuration->databasepath)) {
    DoLog(0) && (Log() << Verbose(0) << "Element list loaded successfully." << endl);
    //periode->Output();
    return Action::success;
  } else {
    DoLog(0) && (Log() << Verbose(0) << "Element list loading failed." << endl);
    return Action::failure;
  }

}

Action::state_ptr CommandElementDbAction::performUndo(Action::state_ptr _state) {
//  ParserLoadXyzState *state = assert_cast<ParserLoadXyzState*>(_state.get());

  return Action::failure;
//  string newName = state->mol->getName();
//  state->mol->setName(state->lastName);
//
//  return Action::state_ptr(new ParserLoadXyzState(state->mol,newName));
}

Action::state_ptr CommandElementDbAction::performRedo(Action::state_ptr _state){
  return Action::failure;
}

bool CommandElementDbAction::canUndo() {
  return false;
}

bool CommandElementDbAction::shouldUndo() {
  return false;
}

const string CommandElementDbAction::getName() {
  return NAME;
}
