/*
 * Project: MoleCuilder
 * Description: creates and alters molecular systems
 * Copyright (C)  2010 University of Bonn. All rights reserved.
 * Please see the LICENSE file or "Copyright notice" in builder.cpp for details.
 */

/*
 * VerboseAction.cpp
 *
 *  Created on: May 9, 2010
 *      Author: heber
 */

// include config.h
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "Helpers/MemDebug.hpp"

#include "Actions/CommandAction/VerboseAction.hpp"
#include "Actions/ActionRegistry.hpp"
#include "Helpers/Log.hpp"
#include "Helpers/Verbose.hpp"

#include <iostream>
#include <string>

using namespace std;

#include "UIElements/UIFactory.hpp"
#include "UIElements/Dialog.hpp"
#include "Actions/ValueStorage.hpp"

// memento to remember the state when undoing

class CommandVerboseState : public ActionState {
public:
  CommandVerboseState(const int _oldverbosity, const int _newverbosity) :
    oldverbosity(_oldverbosity),
    newverbosity(_newverbosity)
  {}
  int oldverbosity;
  int newverbosity;
};


const char CommandVerboseAction::NAME[] = "verbose";

CommandVerboseAction::CommandVerboseAction() :
  Action(NAME)
{}

CommandVerboseAction::~CommandVerboseAction()
{}

void CommandVerbose(int verbosity) {
  ValueStorage::getInstance().setCurrentValue(CommandVerboseAction::NAME, verbosity);
  ActionRegistry::getInstance().getActionByName(CommandVerboseAction::NAME)->call(Action::NonInteractive);
};

void CommandVerboseAction::getParametersfromValueStorage()
{};

Dialog* CommandVerboseAction::fillDialog(Dialog *dialog) {
  ASSERT(dialog,"No Dialog given when filling action dialog");

  dialog->queryInt(NAME, ValueStorage::getInstance().getDescription(NAME));

  return dialog;
}

Action::state_ptr CommandVerboseAction::performCall() {
  int oldverbosity = getVerbosity();
  int newverbosity = 2;

  ValueStorage::getInstance().queryCurrentValue(NAME, newverbosity);

  if (oldverbosity != newverbosity) {
    CommandVerboseState *UndoState = new CommandVerboseState(oldverbosity, newverbosity);
    setVerbosity(newverbosity);
    DoLog(0) && (Log() << Verbose(0) << "Setting verbosity from " << oldverbosity << " to " << newverbosity << "." << endl);
    return Action::state_ptr(UndoState);
  } else {
    DoLog(0) && (Log() << Verbose(0) << "Verbosity remains unchanged at " << oldverbosity << "." << endl);
    return Action::failure;
  }
}

Action::state_ptr CommandVerboseAction::performUndo(Action::state_ptr _state) {
  CommandVerboseState *state = assert_cast<CommandVerboseState*>(_state.get());

  DoLog(0) && (Log() << Verbose(0) << "Setting verbosity from " << state->newverbosity << " to " << state->oldverbosity << "." << endl);
  setVerbosity(state->oldverbosity);

  return Action::state_ptr(_state);
}

Action::state_ptr CommandVerboseAction::performRedo(Action::state_ptr _state){
  CommandVerboseState *state = assert_cast<CommandVerboseState*>(_state.get());

  DoLog(0) && (Log() << Verbose(0) << "Setting verbosity from " << state->oldverbosity << " to " << state->newverbosity << "." << endl);
  setVerbosity(state->newverbosity);

  return Action::state_ptr(_state);
}

bool CommandVerboseAction::canUndo() {
  return true;
}

bool CommandVerboseAction::shouldUndo() {
  return true;
}

const string CommandVerboseAction::getName() {
  return NAME;
}
